from typing import Any

import requests
from dataclasses import dataclass

from .const import CONTENT_API, ConfiguratorConst
from .common import get_response_json


@dataclass
class Gender:
    name: str


@dataclass
class Country:
    name: str
    fullName: str


@dataclass
class Season:
    name: str


@dataclass
class TNVED:
    subjectName: str
    tnvedName: str
    description: str
    isKiz: bool


def configurator_get_all_genders(ses: requests.Session) -> list[Gender]:
    r = ses.get(CONTENT_API + "/content/v1/directory/kinds", timeout=ConfiguratorConst.TimeoutSec)
    j = get_response_json(r)

    result = [Gender(name=x) for x in j.get("data", [])]

    return result


def configurator_get_all_countries(ses: requests.Session) -> list[Country]:
    r = ses.get(CONTENT_API + "/content/v1/directory/countries", timeout=ConfiguratorConst.TimeoutSec)
    j = get_response_json(r)

    result = [Country(**x) for x in j.get("data", [])]

    return result


def configurator_get_all_seasons(ses: requests.Session) -> list[Season]:
    r = ses.get(CONTENT_API + "/content/v1/directory/seasons", timeout=ConfiguratorConst.TimeoutSec)
    j = get_response_json(r)

    result = [Season(name=x) for x in j.get("data", [])]

    return result


def configurator_get_all_tnveds(ses: requests.Session, category_name: str, tnved_like: str = None) -> list[TNVED]:
    query = {
        "objectName": category_name,
        "tnvedsLike": tnved_like,
    }
    query = {k: v for k, v in query.items() if v}

    r = ses.get(CONTENT_API + "/content/v1/directory/tnved", params=query, timeout=ConfiguratorConst.TimeoutSec)
    j = get_response_json(r)

    result = [TNVED(**x) for x in j.get("data", [])]

    return result


@dataclass
class Limits:
    freeLimits: int
    paidLimits: int


def get_limits(ses: requests.Session) -> Limits:
    r = ses.get(CONTENT_API + "/content/v1/cards/limits", timeout=ConfiguratorConst.TimeoutSec)
    j = get_response_json(r)

    return Limits(**j["data"])


@dataclass
class CardSize:
    chrtID: int
    techSize: str
    skus: list[str]


@dataclass
class Card:
    sizes: list[CardSize]
    mediaFiles: Any
    colors: Any
    updateAt: str
    vendorCode: str
    brand: str
    object: str
    nmID: int
    imtID: int
    objectID: int
    isProhibited: bool
    tags: Any

    def __init__(self, d: dict):
        for k, v in d.items():
            if k == "sizes":
                self.sizes = [CardSize(**x) for x in v]
            else:
                setattr(self, k, v)


def get_all_cards(ses: requests.Session) -> list[Card]:
    result: list[Card] = []
    limit: int = 1_000

    body = {
        "sort": {
            "cursor": {
                "limit": limit
            },
            "filter": {
                "withPhoto": -1
            }
        }
    }

    while True:
        r = ses.post(CONTENT_API + "/content/v1/cards/cursor/list", json=body, timeout=ConfiguratorConst.TimeoutSec)
        j = get_response_json(r)
        cards = j.get("data", []).get("cards", [])
        result.extend(Card(x) for x in cards)
        if len(cards) < limit:
            break
        with body["sort"]["cursor"] as cur:
            cur["updateAt"] = result[-1].updateAt
            cur["nmID"] = result[-1].nmID

    return result


def get_cards_by_vendor_codes(ses: requests.Session, vendor_codes: list[str], allowed_categories_only: bool = False) -> list[Card]:
    body = {
        "vendorCodes": vendor_codes,
        "allowedCategoriesOnly": allowed_categories_only,
    }

    r = ses.post(CONTENT_API + "/content/v1/cards/filter", json=body, timeout=ConfiguratorConst.TimeoutSec)
    j = get_response_json(r)
    return [Card(x) for x in j["data"]]
